package by.zerg.cashoutcalculator.config;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import by.zerg.cashoutcalculator.helpers.ArrayHelper;
import by.zerg.cashoutcalculator.model.ATM;
import by.zerg.cashoutcalculator.model.Card;
import by.zerg.cashoutcalculator.model.Currency;

import static by.zerg.cashoutcalculator.helpers.Constants.*;

public class Config {
	private static List<ATM> atms;
	private static List<Card> cards;

	public static void load(File config) {
		atms = new ArrayList<ATM>();
		cards = new ArrayList<Card>();
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		try {
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document dom = db.parse(config);
			Element root = dom.getDocumentElement();
			parseAtms(root.getElementsByTagName(TAG_ATM));
			parseCards(root.getElementsByTagName(TAG_CARD));

		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	private static void parseAtms(NodeList nodes) {
		if (nodes != null && nodes.getLength() > 0) {
			for (int i = 0; i < nodes.getLength(); i++) {
				Element atmNode = (Element) nodes.item(i);
				String visibleAttribute = atmNode
						.getAttribute(ATTRIBUTE_VISIBLE);
				if (visibleAttribute != null && visibleAttribute.equals("true")) {
					Node nameNode = atmNode.getElementsByTagName(TAG_NAME)
							.item(0);
					String name = nameNode.getTextContent().trim();
					Node currencyNode = atmNode.getElementsByTagName(
							TAG_CURRENCY).item(0);
					Currency currency = Currency.valueOf(currencyNode
							.getTextContent().trim());
					Node loadingNode = atmNode
							.getElementsByTagName(TAG_LOADING).item(0);
					Integer[] loading = ArrayHelper.parse(loadingNode
							.getTextContent().trim());
					atms.add(new ATM(name, currency, loading));
				}
			}
		}
	}

	private static void parseCards(NodeList nodes) {
		if (nodes != null && nodes.getLength() > 0) {
			for (int i = 0; i < nodes.getLength(); i++) {
				Element cardNode = (Element) nodes.item(i);
				String showAttribute = cardNode.getAttribute(ATTRIBUTE_VISIBLE);
				if (showAttribute != null && showAttribute.equals("true")) {
					Node nameNode = cardNode.getElementsByTagName(TAG_NAME)
							.item(0);
					String name = nameNode.getTextContent().trim();
					Node currencyNode = cardNode.getElementsByTagName(
							TAG_CURRENCY).item(0);
					Currency currency = Currency.valueOf(currencyNode
							.getTextContent().trim());
					Node typeNode = cardNode.getElementsByTagName(TAG_TYPE)
							.item(0);
					String type = typeNode.getTextContent().trim();
					Node atmRakeNode = cardNode.getElementsByTagName(
							TAG_ATM_RAKE).item(0);
					double atmRake = atmRakeNode == null ? 0 : Double
							.parseDouble(atmRakeNode.getTextContent().trim());
					Node minAtmRakeNode = cardNode.getElementsByTagName(
							TAG_MIN_ATM_RAKE).item(0);
					double minAtmRake = minAtmRakeNode == null ? 0
							: Double.parseDouble(minAtmRakeNode
									.getTextContent().trim());
					Node maxAtmRakeNode = cardNode.getElementsByTagName(
							TAG_MAX_ATM_RAKE).item(0);
					double maxAtmRake = maxAtmRakeNode == null ? Double.MAX_VALUE
							: Double.parseDouble(maxAtmRakeNode
									.getTextContent().trim());
					Node cardRakeNode = cardNode.getElementsByTagName(
							TAG_CARD_RAKE).item(0);
					double cardRake = cardRakeNode == null ? 0 : Double
							.parseDouble(cardRakeNode.getTextContent().trim());
					Node minCardRakeNode = cardNode.getElementsByTagName(
							TAG_MIN_CARD_RAKE).item(0);
					double minCardRake = minCardRakeNode == null ? 0 : Double
							.parseDouble(minCardRakeNode.getTextContent()
									.trim());
					Node maxCardRakeNode = cardNode.getElementsByTagName(
							TAG_MAX_CARD_RAKE).item(0);
					double maxCardRake = maxCardRakeNode == null ? Double.MAX_VALUE
							: Double.parseDouble(maxCardRakeNode
									.getTextContent().trim());
					Node systemRakeNode = cardNode.getElementsByTagName(
							TAG_SYSTEM_RAKE).item(0);
					double systemRake = systemRakeNode == null ? 0
							: Double.parseDouble(systemRakeNode
									.getTextContent().trim());
					Node minSystemRakeNode = cardNode.getElementsByTagName(
							TAG_MIN_SYSTEM_RAKE).item(0);
					double minSystemRake = minSystemRakeNode == null ? 0
							: Double.parseDouble(minSystemRakeNode
									.getTextContent().trim());
					Node maxSystemRakeNode = cardNode.getElementsByTagName(
							TAG_MAX_SYSTEM_RAKE).item(0);
					double maxSystemRake = maxSystemRakeNode == null ? Double.MAX_VALUE
							: Double.parseDouble(maxSystemRakeNode
									.getTextContent().trim());
					cards.add(new Card(name, currency, type, atmRake,
							minAtmRake, maxAtmRake, cardRake, minCardRake,
							maxCardRake, systemRake, minSystemRake,
							maxSystemRake));
				}
			}
		}
	}

	public static List<ATM> getAtms() {
		return atms;
	}

	public static List<Card> getCards() {
		return cards;
	}
}
