package by.zerg.cashoutcalculator.gui;

import java.awt.LayoutManager;
import java.util.Map;

import javax.swing.JPanel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.CategoryLabelPositions;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.labels.StandardCategoryItemLabelGenerator;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.DatasetRenderingOrder;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.CategoryItemRenderer;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;

import by.zerg.cashoutcalculator.helpers.Constants;

import static by.zerg.cashoutcalculator.helpers.Constants.*;

public class GraphPanel extends JPanel {
	private Map<String, Map<Double, Double>> data;

	public GraphPanel(LayoutManager lm) {
		super(lm);
	}

	public void setData(Map<String, Map<Double, Double>> data) {
		this.data = data;
	}

	private CategoryDataset createDataset() {
		final DefaultCategoryDataset dataset = new DefaultCategoryDataset();
		for (String cardName : data.keySet()) {
			Map<Double, Double> values = data.get(cardName);
			for (Double amount : values.keySet()) {
				Double commision = values.get(amount);
				dataset.setValue(commision, cardName, amount);
			}
		}
		return dataset;
	}

	private JFreeChart createChart(CategoryDataset categoryDataset) {
		JFreeChart barChart = ChartFactory.createBarChart(CHART_TITLE,
				CHART_LABEL_X, CHART_LABEL_Y, categoryDataset,
				PlotOrientation.VERTICAL, true, true, false);
		CategoryPlot categoryPlot = (CategoryPlot) barChart.getPlot();
		CategoryAxis categoryAxis = categoryPlot.getDomainAxis();
		categoryAxis.setLowerMargin(0.02);
		categoryAxis.setUpperMargin(0.02);
		categoryAxis.setCategoryLabelPositions(CategoryLabelPositions.UP_45);
		NumberAxis numberAxis = (NumberAxis) categoryPlot.getRangeAxis();
		numberAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
		numberAxis.setUpperMargin(0.1);
		CategoryItemRenderer renderer = categoryPlot.getRenderer();
		renderer.setBaseItemLabelGenerator(new StandardCategoryItemLabelGenerator(
				"{2}", Constants.DECIMAL_FORMATTER, Constants.DECIMAL_FORMATTER));
		renderer.setBaseItemLabelsVisible(true);
		categoryPlot.mapDatasetToRangeAxis(1, 1);
		categoryPlot.setDatasetRenderingOrder(DatasetRenderingOrder.FORWARD);
		ChartUtilities.applyCurrentTheme(barChart);
		return barChart;
	}

	public void refresh() {
		CategoryDataset categoryDataset = createDataset();
		JFreeChart chart = createChart(categoryDataset);
		setVisible(false);
		removeAll();
		add(new ChartPanel(chart));
		setVisible(true);
	}
}
