package by.zerg.cashoutcalculator.gui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Comparator;
import java.util.Map;
import java.util.TreeMap;

import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import by.zerg.cashoutcalculator.config.Config;
import by.zerg.cashoutcalculator.helpers.Constants;
import by.zerg.cashoutcalculator.model.ATM;
import by.zerg.cashoutcalculator.model.Card;
import by.zerg.cashoutcalculator.model.CashoutCalculator;
import by.zerg.cashoutcalculator.model.CashoutCalculatorImpl;

import static by.zerg.cashoutcalculator.helpers.Constants.*;

public class MainFrame extends JFrame implements ActionListener {
	public static final int VIEW_TABLE = 0;
	public static final int VIEW_GRAPH = 1;

	private CashoutCalculator calculator;
	private JTextField cashoutTextField;
	private JTextField resultTextField;
	private JTextField comissionTextFiled;
	private JButton calculateButton;
	private JTextArea examplesTextArea;
	private JRadioButton tableViewRadioButton;
	private JRadioButton graphViewRadioButton;
	private JComboBox cardsList;
	private JComboBox atmsList;
	private GraphPanel graphPanel;
	private JPanel dataPanel;
	private JTextField startAmount;
	private JTextField endAmount;
	private JTextField rangeAmount;
	private JCheckBox[] cardsCheckboxes;
	private JButton goButton;

	public MainFrame() {
		calculator = new CashoutCalculatorImpl();
		cashoutTextField = new JTextField();
		cashoutTextField.setHorizontalAlignment(JTextField.RIGHT);
		resultTextField = new JTextField();
		resultTextField.setHorizontalAlignment(JTextField.RIGHT);
		resultTextField.setEditable(false);
		comissionTextFiled = new JTextField();
		comissionTextFiled.setHorizontalAlignment(JTextField.RIGHT);
		comissionTextFiled.setEditable(false);
		calculateButton = new JButton(LABEL_CALCULATE);
		calculateButton.setActionCommand(LABEL_CALCULATE);
		calculateButton.addActionListener(this);
		examplesTextArea = new JTextArea(10, 0);
		examplesTextArea.setEditable(false);
		graphPanel = new GraphPanel(new BorderLayout());
		tableViewRadioButton = new JRadioButton(LABEL_TABLE_VIEW);
		tableViewRadioButton.setActionCommand(LABEL_TABLE_VIEW);
		tableViewRadioButton.setSelected(true);
		ActionListener radioListener = new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				String actionCommand = e.getActionCommand();
				int view = actionCommand.equals(LABEL_TABLE_VIEW) ? VIEW_TABLE
						: VIEW_GRAPH;
				switchView(view);
			}
		};
		tableViewRadioButton.addActionListener(radioListener);

		graphViewRadioButton = new JRadioButton(LABEL_GRAPH_VIEW);
		graphViewRadioButton.addActionListener(radioListener);

		atmsList = new JComboBox(Config.getAtms().toArray());
		atmsList.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				fillData();
			}
		});

		cardsList = new JComboBox(Config.getCards().toArray());
		cardsList.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				Card card = (Card) cardsList.getSelectedItem();
				for (JCheckBox cb : cardsCheckboxes) {
					cb.setSelected(cb.getText().equals(card.getName()));
				}
				fillData();
			}
		});

		startAmount = new JTextField(DEFAULT_START_AMOUNT, 3);
		startAmount.setHorizontalAlignment(JTextField.RIGHT);
		endAmount = new JTextField(DEFAULT_END_AMOUNT, 3);
		endAmount.setHorizontalAlignment(JTextField.RIGHT);
		rangeAmount = new JTextField(DEFAULT_RANGE_AMOUNT, 3);
		rangeAmount.setHorizontalAlignment(JTextField.RIGHT);
		goButton = new JButton(LABEL_GO);
		goButton.setActionCommand(LABEL_GO);
		goButton.addActionListener(this);

		cardsCheckboxes = new JCheckBox[Config.getCards().size()];
		int i = 0;
		for (Card card : Config.getCards()) {
			cardsCheckboxes[i] = new JCheckBox(card.getName());
			cardsCheckboxes[i++].addActionListener(new ActionListener() {

				@Override
				public void actionPerformed(ActionEvent e) {
					fillData();
				}
			});
		}
		cardsCheckboxes[0].setSelected(true);
	}

	public void init() {
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setTitle(TITLE);
		setLayout(new BorderLayout());

		JPanel leftPanel = new JPanel();
		Box leftBox = Box.createVerticalBox();

		JPanel cashoutPanel = new JPanel(new BorderLayout());
		cashoutPanel.setBorder(new TitledBorder(LABEL_CASHOUT));
		cashoutPanel.add(cashoutTextField);
		leftBox.add(cashoutPanel);

		JPanel cardsListPanel = new JPanel(new BorderLayout());
		cardsListPanel.setBorder(new TitledBorder(LABEL_CARD));
		cardsListPanel.add(cardsList);
		leftBox.add(cardsListPanel);

		JPanel transactionAmountsPanel = new JPanel(new BorderLayout());
		transactionAmountsPanel.setBorder(new TitledBorder(LABEL_ATM));
		transactionAmountsPanel.add(atmsList);
		leftBox.add(transactionAmountsPanel);

		JPanel calculateButtonPanel = new JPanel(new FlowLayout());
		calculateButtonPanel.add(calculateButton);
		leftBox.add(calculateButtonPanel);

		JPanel resultPanel = new JPanel(new BorderLayout());
		resultPanel.setBorder(new TitledBorder(LABEL_AMOUNT));
		resultPanel.add(resultTextField);
		leftBox.add(resultPanel);

		JPanel comissionPanel = new JPanel(new BorderLayout());
		comissionPanel.setBorder(new TitledBorder(LABEL_COMISSION));
		comissionPanel.add(comissionTextFiled);
		leftBox.add(comissionPanel);

		leftPanel.add(leftBox);
		add(leftPanel, BorderLayout.WEST);

		JPanel rightPanel = new JPanel(new BorderLayout());

		JPanel cardsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		for (JCheckBox cb : cardsCheckboxes) {
			cardsPanel.add(cb);
		}
		rightPanel.add(cardsPanel, BorderLayout.NORTH);

		dataPanel = new JPanel(new BorderLayout());
		rightPanel.add(dataPanel);
		add(rightPanel);

		JPanel rangePanel = new JPanel();
		rangePanel.add(new JLabel(LABEL_START_AMOUNT));
		rangePanel.add(startAmount);
		rangePanel.add(new JLabel(LABEL_END_AMOUNT));
		rangePanel.add(endAmount);
		rangePanel.add(new JLabel(LABEL_RANGE_AMOUNT));
		rangePanel.add(rangeAmount);
		rangePanel.add(goButton);
		rightPanel.add(rangePanel, BorderLayout.SOUTH);

		fillData();
		switchView(VIEW_TABLE);

		JPanel bottomPanel = new JPanel();
		ButtonGroup radioButtonGroup = new ButtonGroup();
		radioButtonGroup.add(tableViewRadioButton);
		radioButtonGroup.add(graphViewRadioButton);
		bottomPanel.add(tableViewRadioButton);
		bottomPanel.add(graphViewRadioButton);
		add(bottomPanel, BorderLayout.SOUTH);

		pack();
		setLocationRelativeTo(null);
		setVisible(true);
	}

	private void switchView(int view) {
		dataPanel.removeAll();
		dataPanel.setVisible(false);
		switch (view) {
		case VIEW_TABLE:
			dataPanel.add(new JScrollPane(examplesTextArea));
			break;
		case VIEW_GRAPH:
			dataPanel.add(graphPanel);
			break;
		}
		dataPanel.setVisible(true);
	}

	private void fillData() {
		double start = Double.valueOf(DEFAULT_START_AMOUNT);
		double end = Double.valueOf(DEFAULT_END_AMOUNT);
		double range = Double.valueOf(DEFAULT_RANGE_AMOUNT);
		try {
			start = Double.valueOf(startAmount.getText());
			end = Double.valueOf(endAmount.getText());
			range = Double.valueOf(rangeAmount.getText());
		} catch (Throwable t) {
			// TODO Auto-generated catch block
			t.printStackTrace();
		}
		calculator.setATM((ATM) atmsList.getSelectedItem());
		Map<String, Map<Double, Double>> data = new TreeMap<String, Map<Double, Double>>(
				new Comparator<String>() {

					@Override
					public int compare(String s1, String s2) {
						return s1.toLowerCase().compareTo(s2.toLowerCase());
					}
				});
		examplesTextArea.setText(null);
		for (JCheckBox cb : cardsCheckboxes) {
			if (cb.isSelected()) {
				Card card = Card.valueOf(cb.getText());
				calculator.setCard(card);
				Map<Double, Double> values = new TreeMap<Double, Double>();
				examplesTextArea.append("*** " + card + " ***\n\n"
						+ "USD\tWMZ\t%");
				for (double i = start; i <= end; i += range) {
					double result = Math.ceil(calculator.calculate(i) * 100) / 100;
					double commision = calculator.calculateCommissionRate(i) * 100;
					if (i == start) {
						examplesTextArea.append("\n");
					}
					examplesTextArea.append(i + "\t" + result + "\t"
							+ Constants.DECIMAL_FORMATTER.format(commision)
							+ "\n");
					if (i == end) {
						examplesTextArea.append("\n");
					}
					values.put((double) i, commision);
				}
				data.put(card.getName(), values);
			}
		}
		examplesTextArea.setCaretPosition(0);
		graphPanel.setData(data);
		graphPanel.refresh();
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String actionCommand = e.getActionCommand();
		if (actionCommand.equals(LABEL_CALCULATE)) {
			double amount = Double.parseDouble(cashoutTextField.getText());
			calculator.setATM((ATM) atmsList.getSelectedItem());
			calculator.setCard((Card) cardsList.getSelectedItem());
			double result = calculator.calculate(amount);
			resultTextField
					.setText(String.valueOf(Math.ceil(result * 100) / 100));
			double comission = calculator.calculateCommissionRate(amount);
			comissionTextFiled.setText(Constants.DECIMAL_FORMATTER
					.format(comission * 100));
		}
		fillData();
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		Config.load(new File(DEFAULT_CONFIG));
		new MainFrame().init();
	}
}
